@@ -16,7 +16,7 @@ validity of the host key.
 */
 
 #include "includes.h"
-RCSID("$Id: auth-rsa.c,v 1.6 1999/10/31 12:31:25 bg Exp $");
+RCSID("$Id: auth-rsa.c,v 1.7 2000/02/28 17:53:11 bg Exp $");
 
 #include "rsa.h"
 #include "randoms.h"
@@ -402,7 +402,24 @@ int auth_rsa(struct passwd *pw, BIGNUM *client_n, RandomState *state, int strict
 		      packet_send_debug("Your host '%.200s' is not permitted to use this key for login.",
 					get_canonical_hostname());
 		      xfree(patterns);
+		      /* key invalid for this host, reset flags */
 		      authenticated = 0;
+		      no_agent_forwarding_flag = 0;
+		      no_port_forwarding_flag = 0;
+		      no_pty_flag = 0;
+		      no_x11_forwarding_flag = 0;
+		      while (custom_environment)
+			{
+			  struct envstring *ce = custom_environment;
+			  custom_environment = ce->next;
+			  xfree(ce->s);
+			  xfree(ce);
+			}
+		      if (forced_command)
+			{
+			  xfree(forced_command);
+			  forced_command = NULL;
+			}
 		      break;
 		    }
 		  xfree(patterns);
